import { createSpy } from 'expect';
import createFieldProps from '../createFieldProps';
import plain from '../structure/plain';
import plainExpectations from '../structure/plain/expectations';
import immutable from '../structure/immutable';
import immutableExpectations from '../structure/immutable/expectations';
import addExpectations from './addExpectations';

var describeCreateFieldProps = function describeCreateFieldProps(name, structure, expect) {
  var empty = structure.empty;
  var getIn = structure.getIn;
  var fromJS = structure.fromJS;


  describe(name, function () {
    it('should pass value through', function () {
      expect(createFieldProps(getIn, 'foo', { value: 'hello' }).value).toBe('hello');
    });

    it('should calculate dirty/pristine', function () {
      expect(createFieldProps(getIn, 'foo', { initial: 'bar', value: 'bar' }).dirty).toBe(false);
      expect(createFieldProps(getIn, 'foo', { initial: 'bar', value: 'bar' }).pristine).toBe(true);
      expect(createFieldProps(getIn, 'foo', { initial: 'bar', value: 'baz' }).dirty).toBe(true);
      expect(createFieldProps(getIn, 'foo', { initial: 'bar', value: 'baz' }).pristine).toBe(false);
    });

    it('should provide onBlur', function () {
      var blur = createSpy();
      expect(blur).toNotHaveBeenCalled();
      var result = createFieldProps(getIn, 'foo', { initial: 'bar', value: 'bar', blur: blur });
      expect(result.onBlur).toBeA('function');
      expect(blur).toNotHaveBeenCalled();
      result.onBlur('rabbit');
      expect(blur).toHaveBeenCalled().toHaveBeenCalledWith('rabbit');
    });

    it('should provide onChange', function () {
      var change = createSpy();
      expect(change).toNotHaveBeenCalled();
      var result = createFieldProps(getIn, 'foo', { initial: 'bar', value: 'bar', change: change });
      expect(result.onChange).toBeA('function');
      expect(change).toNotHaveBeenCalled();
      result.onChange('rabbit');
      expect(change).toHaveBeenCalled().toHaveBeenCalledWith('rabbit');
    });

    it('should provide onUpdate alias for onChange', function () {
      var result = createFieldProps(getIn, 'foo', { initial: 'bar', value: 'bar' });
      expect(result.onUpdate).toBe(result.onChange);
    });

    it('should provide onFocus', function () {
      var focus = createSpy();
      expect(focus).toNotHaveBeenCalled();
      var result = createFieldProps(getIn, 'foo', { initial: 'bar', value: 'bar', focus: focus });
      expect(result.onFocus).toBeA('function');
      expect(focus).toNotHaveBeenCalled();
      result.onFocus('rabbit');
      expect(focus).toHaveBeenCalled().toHaveBeenCalledWith('foo');
    });

    it('should provide onDragStart', function () {
      var result = createFieldProps(getIn, 'foo', { initial: 'bar', value: 'bar' });
      expect(result.onDragStart).toBeA('function');
    });

    it('should provide onDrop', function () {
      var result = createFieldProps(getIn, 'foo', { initial: 'bar', value: 'bar' });
      expect(result.onDrop).toBeA('function');
    });

    it('should read active from state', function () {
      var inactiveResult = createFieldProps(getIn, 'foo', {
        initial: 'bar',
        value: 'bar',
        state: empty
      });
      expect(inactiveResult.active).toBe(false);
      var activeResult = createFieldProps(getIn, 'foo', {
        initial: 'bar',
        value: 'bar',
        state: fromJS({
          active: true
        })
      });
      expect(activeResult.active).toBe(true);
    });

    it('should read touched from state', function () {
      var untouchedResult = createFieldProps(getIn, 'foo', {
        initial: 'bar',
        value: 'bar',
        state: empty
      });
      expect(untouchedResult.touched).toBe(false);
      var touchedResult = createFieldProps(getIn, 'foo', {
        initial: 'bar',
        value: 'bar',
        state: fromJS({
          touched: true
        })
      });
      expect(touchedResult.touched).toBe(true);
    });

    it('should read visited from state', function () {
      var notVisitedResult = createFieldProps(getIn, 'foo', {
        initial: 'bar',
        value: 'bar',
        state: empty
      });
      expect(notVisitedResult.visited).toBe(false);
      var visitedResult = createFieldProps(getIn, 'foo', {
        initial: 'bar',
        value: 'bar',
        state: fromJS({
          visited: true
        })
      });
      expect(visitedResult.visited).toBe(true);
    });

    it('should read sync errors from param', function () {
      var noErrorResult = createFieldProps(getIn, 'foo', {
        initial: 'bar',
        value: 'bar',
        state: empty
      });
      expect(noErrorResult.error).toNotExist();
      expect(noErrorResult.valid).toBe(true);
      expect(noErrorResult.invalid).toBe(false);
      var errorResult = createFieldProps(getIn, 'foo', {
        initial: 'bar',
        value: 'bar',
        state: empty
      }, 'This is an error');
      expect(errorResult.error).toBe('This is an error');
      expect(errorResult.valid).toBe(false);
      expect(errorResult.invalid).toBe(true);
    });

    it('should read async errors from state', function () {
      var noErrorResult = createFieldProps(getIn, 'foo', {
        initial: 'bar',
        value: 'bar',
        state: empty
      });
      expect(noErrorResult.error).toNotExist();
      expect(noErrorResult.valid).toBe(true);
      expect(noErrorResult.invalid).toBe(false);
      var errorResult = createFieldProps(getIn, 'foo', {
        initial: 'bar',
        value: 'bar',
        state: empty
      }, 'This is an error');
      expect(errorResult.error).toBe('This is an error');
      expect(errorResult.valid).toBe(false);
      expect(errorResult.invalid).toBe(true);
    });

    it('should read submit errors from state', function () {
      var noErrorResult = createFieldProps(getIn, 'foo', {
        initial: 'bar',
        value: 'bar',
        state: empty
      });
      expect(noErrorResult.error).toNotExist();
      expect(noErrorResult.valid).toBe(true);
      expect(noErrorResult.invalid).toBe(false);
      var errorResult = createFieldProps(getIn, 'foo', {
        initial: 'bar',
        value: 'bar',
        state: empty,
        submitError: 'This is an error'
      });
      expect(errorResult.error).toBe('This is an error');
      expect(errorResult.valid).toBe(false);
      expect(errorResult.invalid).toBe(true);
    });

    it('should prioritize sync errors over async or submit errors', function () {
      var noErrorResult = createFieldProps(getIn, 'foo', {
        initial: 'bar',
        value: 'bar',
        state: empty
      });
      expect(noErrorResult.error).toNotExist();
      expect(noErrorResult.valid).toBe(true);
      expect(noErrorResult.invalid).toBe(false);
      var errorResult = createFieldProps(getIn, 'foo', {
        initial: 'bar',
        value: 'bar',
        asyncError: 'async error',
        submitError: 'submit error'
      }, 'sync error');
      expect(errorResult.error).toBe('sync error');
      expect(errorResult.valid).toBe(false);
      expect(errorResult.invalid).toBe(true);
    });

    it('should pass through other props', function () {
      var result = createFieldProps(getIn, 'foo', {
        initial: 'bar',
        value: 'bar',
        state: empty,
        someOtherProp: 'dog',
        className: 'my-class'
      });
      expect(result.initial).toNotExist();
      expect(result.state).toNotExist();
      expect(result.someOtherProp).toBe('dog');
      expect(result.className).toBe('my-class');
    });

    it('should set checked for checkboxes', function () {
      expect(createFieldProps(getIn, 'foo', {
        state: empty,
        type: 'checkbox'
      }).checked).toBe(false);
      expect(createFieldProps(getIn, 'foo', {
        value: true,
        state: empty,
        type: 'checkbox'
      }).checked).toBe(true);
      expect(createFieldProps(getIn, 'foo', {
        value: false,
        state: empty,
        type: 'checkbox'
      }).checked).toBe(false);
    });

    it('should set checked for radio buttons', function () {
      expect(createFieldProps(getIn, 'foo', {
        state: empty,
        type: 'radio',
        _value: 'bar'
      }).checked).toBe(false);
      expect(createFieldProps(getIn, 'foo', {
        value: 'bar',
        state: empty,
        type: 'radio',
        _value: 'bar'
      }).checked).toBe(true);
      expect(createFieldProps(getIn, 'foo', {
        value: 'baz',
        state: empty,
        type: 'radio',
        _value: 'bar'
      }).checked).toBe(false);
    });

    it('should default value to [] for multi-selects', function () {
      expect(createFieldProps(getIn, 'foo', {
        state: empty,
        type: 'select-multiple'
      }).value).toBeA('array').toEqual([]);
    });

    it('should replace undefined value with empty string', function () {
      var result = createFieldProps(getIn, 'foo', {
        state: empty
      });
      expect(result.value).toBe('');
    });

    it('should replace undefined value with default value provided', function () {
      var defaultValue = {
        foo: 'some complex value'
      };
      var result = createFieldProps(getIn, 'foo', {
        state: empty
      }, undefined, undefined, defaultValue);
      expect(result.value).toBe(defaultValue);
    });
  });
};

describeCreateFieldProps('createFieldProps.plain', plain, addExpectations(plainExpectations));
describeCreateFieldProps('createFieldProps.immutable', immutable, addExpectations(immutableExpectations));